package kr.co.uplus.ez.api.riskMgt;

import kr.co.uplus.ez.api.riskMgt.dto.*;
import org.apache.ibatis.annotations.Mapper;

import java.util.List;

@Mapper
public interface RiskMgtMapper {

    int selectSndrnoBlckTotalCnt(SendNumIntrcpListReqDto sendNumIntrcpListReqDto);

    List<SendNumIntrcpList> selectSndrnoBlckList(SendNumIntrcpListReqDto sendNumIntrcpListReqDto);

    SendNumIntrcpDetail selectSndrnoBlckDetail(String blckSndrno);

    void insertSndrnoBlck(SendNumInsertIntrcpReqDto sendNumInsertIntrcpReqDto);

    int updateSndrnoBlck(SendNumUpdateIntrcplReqDto sendNumUpdateIntrcplReqDto);

    int deleteSndrnoBlck(SendNumDeleteIntrcpReqDto sendNumDeleteIntrcpReqDto);

    int selectRcvblck080ListTotalCnt(ZezNumIntrcpListReqDto zezNumIntrcpListReqDto);

    List<ZezNumIntrcpList> selectRcvblck080List(ZezNumIntrcpListReqDto zezNumIntrcpListReqDto);

    int selectMsgBlckwordListTotalCnt(MsgIntrcpListReqDto msgIntrcpListReqDto);

    List<MsgIntrcpList> selectMsgBlckwordList(MsgIntrcpListReqDto msgIntrcpListReqDto);

    MsgIntrcpDetail selectMsgBlckwordDetail(MsgIntrcpDetailReqDto msgIntrcpDetailReqDto);

    List<MsgBlckwordList> selectBlckwordDtlList(MsgIntrcpDetailReqDto msgIntrcpDetailReqDto);

    void insertMsgBlckword(MsgInsertIntrcpReqDto msgInsertIntrcpReqDto);

    String selectMsgBlckWordSeq();
    
    void insertBlckwordDtl(List<MsgBlckwordList> msgBlckwordLists);

    int updateMsgBlckword(MsgUpdateIntrcplReqDto msgUpdateIntrcplReqDto);

    int deleteMsgBlckword(MsgDeleteIntrcpReqDto msgDeleteIntrcpReqDto);

    int deleteBlckwordDtl(String seqNo);

    int selectWebsndBlckListTotalCnt(WebInsertIntrcpReqDto webInsertIntrcpReqDto);

    List<WebIntrcpList> selectWebsndBlckList(WebInsertIntrcpReqDto webInsertIntrcpReqDto);

}
